module DEIS
  require "json"
  class Storage < Base
    BASE_PATH = "$HOME/.deis/gem/#{DEIS::VERSION}/"
    ENV_VAR_PATH = DEIS::Storage::BASE_PATH+".profile"

    def self.file(repo, env)
      BASE_PATH.gsub("$HOME", ENV['HOME'])+repo+"/#{env}.json"
    end

    def self.load(repo, env)
      data = {}
      data = JSON.parse( File.open( DEIS::Storage.file(repo, env) , "r" ) { | f | f.read } ) if File.exists? (DEIS::Storage.file(repo, env))
      data
    end

    def self.save(repo, env, data)
      working_dir = DEIS::Storage::BASE_PATH.gsub("$HOME", ENV['HOME'])
      FileUtils.mkdir_p(working_dir+repo) unless Dir.exists?(working_dir+repo)
      File.open( DEIS::Storage.file(repo, env) , "w" ){ |f| f.write ( data.to_json ) }
    end

    def self.get(repo, env, key=nil)
      current = DEIS::Storage.load(repo, env)
      # key asked for and is present
      if ! key.nil? && current.has_key?(key)
        current[key].sort_by{|k,v| k }
      # key asked for but not present
      elsif !key.nil?
        {}
      # key not asked for
      else
        current.sort_by{|k,v| k }
      end
    end

    # remove the removed flag for this key by just setting it to an empty hash
    def self.removal_complete(repo, env, key)
      DEIS::Storage.set(repo, env, key, {})
    end

    # only handles top level, not sub levels
    def self.set(repo, env, key, value)
      current = DEIS::Storage.load(repo, env)
      # overwrite the value existing
      current[key] = value
      DEIS::Storage.save(repo, env, current)
    end

    def self.subset(repo, env, key, subkey, value)
      current = DEIS::Storage.load(repo, env)
      current[key] = {} if ! current.has_key?(key)
      current[key][subkey] = value
      # if it exists in the unset, remove it
      DEIS::Storage.save(repo, env, current)
      # remove from items that need to be removed
      DEIS::Storage.subunset(repo, env, "removed_#{key}", subkey) if key.index("removed_").nil?
    end

    # only have the option to remove low level, not top level
    def self.subunset(repo, env, key, subkey)
      current = DEIS::Storage.load(repo, env)
      if current.has_key?(key) && current[key].has_key?(subkey)
        tmp = current[key]
        val = tmp[subkey]
        tmp.delete(subkey)
        current[key] = tmp
        # save
        DEIS::Storage.save(repo, env, current)
        #also need to store removals for actions to be performed on the server, if check to stop recursive loops
        DEIS::Storage.subset(repo, env, "removed_#{key}", subkey, val) if key.index("removed_").nil?
      end
    end

  end
end